//
// "$Id$"
//
// Fl_Rect header file for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2017 by Bill Spitzak and others.
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     http://www.fltk.org/COPYING.php
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

#ifndef Fl_Rect_H
#define Fl_Rect_H

#include <stdio.h>	// test only: Fl_Rect::print()

/**
  Rectangle with non-standard FLTK coordinates (L, R, T, B).

  This kind of rectangle is used in Fl_Group's sizes() array
  to describe the stored sizes of:

    -# the group itself
    -# the resizable widget
    -# all its children.

  \note	This structure (class) must not be changed for backwards compatibility
	with old programs that access the sizes() array in derived classes.
	Its size is fixed (sizeof(int) * 4), virtual methods are not allowed
	to stay binary compatible.

  Fl_Rect2 can also be used to describe widget coordinates because
  it can be implicitly converted to an Fl_Rect. This is just for convenience.

  \see Fl_Rect
*/

class Fl_Rect2 {

  int left_;
  int right_;
  int top_;
  int bottom_;

public:

  Fl_Rect2()
  : left_(0), right_(0), top_(0), bottom_(0) {}

  Fl_Rect2(int R, int B)
  : left_(0), right_(R), top_(0), bottom_(B) {}

  Fl_Rect2(int L, int R, int T, int B)
  : left_(L), right_(R), top_(T), bottom_(B) {}

  int left()   const { return left_; }			// default accessor
  int right()  const { return right_; }			// default accessor
  int top()    const { return top_; }			// default accessor
  int bottom() const { return bottom_; }		// default accessor

  int x() const { return left_; }			// FLTK standard
  int y() const { return top_; }			// FLTK standard
  int w() const { return right_ - left_; }		// FLTK standard
  int h() const { return bottom_ - top_; }		// FLTK standard

}; // class Fl_Rect2


/**
  Rectangle with standard FLTK coordinates (X, Y, W, H).

  This may be used internally, for overloaded widget contructors and other
  overloaded methods like fl_measure(), fl_text_extents(), fl_rect(),
  fl_rectf(), and maybe more.
*/

class Fl_Rect {

  int x_;
  int y_;
  int w_;
  int h_;

public:

  Fl_Rect()
  : x_(0), y_(0), w_(0), h_(0) {}

  Fl_Rect(int W, int H)
  : x_(0), y_(0), w_(W), h_(H) {}

  Fl_Rect(int X, int Y, int W, int H)
  : x_(X), y_(Y), w_(W), h_(H) {}

  // copy constructor
  Fl_Rect (const Fl_Rect& r)
  : x_(r.x()), y_(r.y()), w_(r.w()), h_(r.h()) {}

  // Fl_Rect constructor (Fl_Rect2 by const reference)	** test only ? **

  Fl_Rect (const Fl_Rect2& r2)
  : x_(r2.x()), y_(r2.y()), w_(r2.w()), h_(r2.h()) {}

  // Fl_Rect constructor (Fl_Rect2 * by const pointer)	** test only ? **

  Fl_Rect (const Fl_Rect2* const r2)
  : x_(r2->x()), y_(r2->y()), w_(r2->w()), h_(r2->h()) {}

  // assignment operator
  Fl_Rect& operator=( Fl_Rect r ) {
    x_ = r.x(); y_ = r.y();
    w_ = r.w(); h_ = r.h();
    return *this;
  }

  // print method					** test only ! **
  void print(const char *t = "???") const {
    printf("%s: x=%d, y=%d, w=%d, h=%d\n", t, x(), y(), w(), h());
  }

  int x() const { return x_; }				// FLTK standard
  int y() const { return y_; }				// FLTK standard
  int w() const { return w_; }				// FLTK standard
  int h() const { return h_; }				// FLTK standard

  int left()   const { return x_; }			// conveniency == x()
  int top()    const { return y_; }			// conveniency == y()
  int right()  const { return x_ + w_; }		// maybe -> r() ?
  int bottom() const { return y_ + h_; }		// maybe -> b() ?


// methods that need Fl_Rect2:


}; // class Fl_Rect

#endif // Fl_Rect_H

//
// End of "$Id$".
//
